/*
 *  BrStatusMonitor.js
 *	
 *  Copyright (c) 2005 Brother Industries, Ltd. All rights reserved.
 *
 */

var gAutoRefreshInterval = 60000;	// 60[sec]
var gTargetIP = null;
var gGetStatusTimer = null;
var gDeviceStatus = {errCode:0, strStatus:"", nSupp_B:0, nSupp_C:0, nSupp_M:0, nSupp_Y:0, isUSB:false, hasEWS:false, devName:"", devLoc:"", modelType:0, panelID:2, nCode:10001, message:""};
var gStatusOfAppli = {isGettingStatus:0, isRotatingRefresh:0, cancelFlag:0}

function getLocalizedStatusMessage (key)
{
	try {
		return localizedStatusMessage[key];
	} catch (ex) {}

	return key;
}

function getLocalizedString (key)
{
	try {
		return localizedStrings[key];
	} catch (ex) {}

	return key;
}

function initStatus(devStat)
{
	devStat.errCode = 0;
	devStat.nCode = 10001;
	devStat.message = "";
	devStat.nSupp_B = 0;
	devStat.nSupp_C = 0;
	devStat.nSupp_M = 0;
	devStat.nSupp_Y = 0;
	devStat.isUSB = false;
	devStat.hasEWS = false;
	devStat.devName = "";
	devStat.devLoc = "";
	devStat.modelType = 0;
	devStat.panelID = 2;
}

function parserOfStatusArray(statArray)
{
	var tempStatus = {errCode:0, strStatus:"", nSupp_B:0, nSupp_C:0, nSupp_M:0, nSupp_Y:0, isUSB:true, hasEWS:false, devName:"", devLoc:"", modelType:0, panelID:2, nCode:10001, message:""};

	for(i=0; i < statArray.length; i++){
		var subArray = statArray[i];
		
		switch(subArray[0]){
			case "errCode": {
				if(subArray[1] == null){
					tempStatus.errCode = 1;
				}
				else{
					tempStatus.errCode = parseInt(subArray[1]);
				}
			}	break;
			
			case "code": {
				if(subArray[1] == null){
					tempStatus.errCode = 1;
				}
				else{
					tempStatus.nCode = subArray[1];
				}
			}	break;
			
			case "message": {
				if(subArray[1] == null){
					tempStatus.errCode = 1;
				}
				else{
					tempStatus.message = getLocalizedStatusMessage(subArray[1]);
				}
			}	break;
			
			case "supp": {
				if(subArray[1] == null){
					tempStatus.errCode = 1;
				}
				else if(subArray[2] == null){
					tempStatus.errCode = 1;
				}
				else if(subArray[3] == null){
					tempStatus.errCode = 1;
				}
				else if(subArray[4] == null){
					tempStatus.errCode = 1;
				}
				else{
					tempStatus.nSupp_B = parseInt(subArray[1]);
					tempStatus.nSupp_C = parseInt(subArray[2]);
					tempStatus.nSupp_M = parseInt(subArray[3]);
					tempStatus.nSupp_Y = parseInt(subArray[4]);
				}
			}	break;
			
			case "isUSB": {
				if(subArray[1] == null){
					tempStatus.errCode = 1;
				}
				else{
					tempStatus.isUSB = new Boolean(subArray[1]);
				}
			}	break;
			
			case "hasEWS": {
				if(subArray[1] == null){
					tempStatus.errCode = 1;
				}
				else{
					tempStatus.hasEWS = new Boolean(subArray[1]);
				}
			}	break;
			
			case "devName": {
				if(subArray[1] == null){
					tempStatus.errCode = 1;
				}
				else if(subArray[2] == null){
					tempStatus.errCode = 1;
				}
				else{
					tempStatus.devName = subArray[1];
					tempStatus.devLoc = subArray[2];
				}
			}	break;
			
			case "modelType": {
				if(subArray[1] == null){
					tempStatus.errCode = 1;
				}
				else{
					tempStatus.modelType = parseInt(subArray[1]);
				}
			}	break;
			
			case "panelID": {
				if(subArray[1] == null){
					tempStatus.errCode = 1;
				}
				else{
					tempStatus.panelID = parseInt(subArray[1]);
				}
			}	break;
			
		}
	}
	
	if(tempStatus.errCode == 0){
		gDeviceStatus.errCode = tempStatus.errCode;
		gDeviceStatus.nCode = tempStatus.nCode;
		gDeviceStatus.message = tempStatus.message;
		gDeviceStatus.nSupp_B = tempStatus.nSupp_B;
		gDeviceStatus.nSupp_C = tempStatus.nSupp_C;
		gDeviceStatus.nSupp_M = tempStatus.nSupp_M;
		gDeviceStatus.nSupp_Y = tempStatus.nSupp_Y;
		gDeviceStatus.isUSB = tempStatus.isUSB;
		gDeviceStatus.hasEWS = tempStatus.hasEWS;
		gDeviceStatus.devName = tempStatus.devName;
		gDeviceStatus.devLoc = tempStatus.devLoc;
		gDeviceStatus.modelType = tempStatus.modelType;
		gDeviceStatus.panelID = tempStatus.panelID;
	}
	else{
		gDeviceStatus.errCode = tempStatus.errCode;
	}
}

function setInkVolumeItem(id, x) 
{
	var newNum = 0, i =0;
	var inkImg = document.getElementById(id);
	
	/*
		 convert 100 stages into 10 stages.
		 0 -> 0,  1-10 -> 10,  11-20 -> 20, ..., 91-100 -> 100
	*/
	newNum = 0;
	for(i=9; i >= 0; i--){
		if(x > 10*i){
			newNum = 10*i + 10;
			break;
		}
	}
	inkImg.style.clip = "rect("+(83-(newNum*83/100))+", 25, 83, 0)";
}

function setInkVolume(b, y, c, m) 
{
	var hina = "INK_img_";
	
	setInkVolumeItem(hina + "BK", b);
	setInkVolumeItem(hina + "Y", y);
	setInkVolumeItem(hina + "C", c);
	setInkVolumeItem(hina + "M", m);	
}

function changeSignal(sig)
{
	var lcd = document.getElementById("LCD-back");
	
	switch(sig) {
		case "red":
			lcd.src = "Images/LCD_red.png";
			break;
		case "yellow":
			lcd.src = "Images/LCD_yellow.png";
			break;
		case "green":
			lcd.src = "Images/LCD_green.png";
			break;
		default:
			break;
	}
}

function setStatusOfGUI(devStat)
{
	var loc = null;
	var name = null;
	var lcd = null;

	if(devStat.errCode == 0){
		// Name
		name = document.getElementById("devName");
		name.innerHTML = devStat.devName;
		nameBack = document.getElementById("devNameBack");
		nameBack.innerHTML = devStat.devName;
		
		// Location
		loc = document.getElementById("devLoc");
		loc.innerHTML = devStat.devLoc;
		
		// LCD
		lcd = document.getElementById("LCD_txt1");
		lcd.innerHTML = devStat.message;
		
		// Supplies
		setInkVolume(devStat.nSupp_B, devStat.nSupp_Y, devStat.nSupp_C, devStat.nSupp_M);
		
		// Signal
		switch(devStat.panelID){
			case 2:
				changeSignal("green");
				break;
			case 1:
				changeSignal("yellow");
				break;
			default:
				changeSignal("red");
				break;
		}
	}
	else{
		// Name
		name = document.getElementById("devName");
		name.innerHTML = getLocalizedString("Connection error");
		nameBack = document.getElementById("devNameBack");
		nameBack.innerHTML = getLocalizedString("Connection error");
		
		// Location
		loc = document.getElementById("devLoc");
		loc.innerHTML = "";
		
		// LCD
		lcd = document.getElementById("LCD_txt1");
		lcd.innerHTML = "";

		// Signal
		changeSignal("red");
	}
}

function doGetStatus ()
{
	var array = null;
	
	gStatusOfAppli.isGettingStatus = 1;
	
	if(BrStatusMonitor){
		if(gTargetIP != null){
			array = BrStatusMonitor.getStatus(gTargetIP);
		}
		if(array != null){
			parserOfStatusArray(array);
		}
		else{
			gDeviceStatus.errCode = 1;
		}
	}
	
	if(gStatusOfAppli.cancelFlag == 0){
		setStatusOfGUI(gDeviceStatus);
	}
	else{
		gStatusOfAppli.cancelFlag = 0;
	}
	
	gStatusOfAppli.isGettingStatus = 0;
	
	document.getElementById("RefrechButton").src = "Images/Refresh.png";
	gStatusOfAppli.isRotatingRefresh = 0;
}

function getStatus ()
{
	var lcd = null;
	
	document.getElementById("RefrechButton").src = "Images/Refresh_b.gif";
	gStatusOfAppli.isRotatingRefresh = 1;
	
	// LCD
	lcd = document.getElementById("LCD_txt1");
	lcd.innerHTML = getLocalizedString("Getting status...");

	window.setTimeout("doGetStatus()", 1000);	
}

function timerFireProc ()
{
	var lcd = null;
	
	if(gTargetIP == null){
		;
	}
	else if(gStatusOfAppli.isGettingStatus != 0){
		;
	}
	else{
		getStatus();
	}
}

function startAutoRefreshTimer ()
{
	if(gGetStatusTimer == null){
		gGetStatusTimer = window.setInterval("timerFireProc()", gAutoRefreshInterval);
	}
}

function removeAutoRefreshTimer ()
{
	if(gGetStatusTimer != null){
		window.clearInterval(gGetStatusTimer);
	}
	gGetStatusTimer = null;
}

function cancelGetStatus()
{
	var lcd = null;

	gStatusOfAppli.cancelFlag = 1;
	
	// LCD
	lcd = document.getElementById("LCD_txt1");
	lcd.innerHTML = gDeviceStatus.message;
}

function refreshButtonMouseClick (event, tag)
{	
	if(gStatusOfAppli.isGettingStatus == 0){
		if(BrStatusMonitor){
			if(gTargetIP == null){
				gTargetIP = BrStatusMonitor.getCurrentTargetIP();
			}
		}
		getStatus();
	}
}

function go_to_brother (tag)
{
	if (window.widget) {
		if(gStatusOfAppli.isGettingStatus == 1){
			cancelGetStatus();
		}
		switch(tag) {
			case "SolutionsButton":
				widget.openURL("http://solutions.brother.com");
				break;
			default:
				break;
		}
	}
}

function mouseDown (event, tag)
{
	var x = "";
	switch(tag) {
		case "SolutionsButton":
			x = "SolutionsButton";
			break;
		default:
			break;
	}
	if(x){
		go_to_brother(x);
	}
	//event.stopPropagation();
	//event.preventDefault();
}

function mouseUp (event, tag)
{
	var x = "";
	switch(tag) {
		case "SolutionsButton":
			x = "BrSolutionsCenter";
			break;
		default:
			break;
	}
}


function showPrefs(event)
{
	var front = document.getElementById("front");
	var back = document.getElementById("back");
	
	if (window.widget){
		widget.prepareForTransition("ToBack");
	}
	
	front.style.display="none";
	back.style.display="block";
	
	if (window.widget){
		setTimeout ('widget.performTransition();', 0);		
	}
	
	document.getElementById('fliprollie').style.display = 'none';
}

function hidePrefs()
{	
	var front = document.getElementById("front");
	var back = document.getElementById("back");
	
	if (window.widget){
		widget.prepareForTransition("ToFront");
	}
	back.style.display="none";
	front.style.display="block";
	
	if (window.widget){
		setTimeout('widget.performTransition();', 0);
	}
}

function enterflip(event)
{
	document.getElementById('fliprollie').style.display = 'block';
}

function exitflip(event)
{
	document.getElementById('fliprollie').style.display = 'none';
}

function doneMouseClicked()
{
	hidePrefs();
}

function onshow () {
	if(gStatusOfAppli.isRotatingRefresh != 0){
		;
	}
	else if(gStatusOfAppli.isGettingStatus != 0){
		;
	}
	else{
		if(BrStatusMonitor){
			if(BrStatusMonitor.isDeviceChanged()){
				initStatus(gDeviceStatus);
				gTargetIP = BrStatusMonitor.getCurrentTargetIP();
			}
			else if(gTargetIP == null){
				gTargetIP = BrStatusMonitor.getCurrentTargetIP();
			}
		}
		getStatus();
	}
	
	// set timer
	removeAutoRefreshTimer();
	startAutoRefreshTimer();
}

function onhide () {
	if(gStatusOfAppli.isGettingStatus != 0){
		cancelGetStatus();
	}
	
	// remove timer
	removeAutoRefreshTimer();
}

function statusMonitorLoaded ()
{
	var doneButton = document.getElementById('done');
	
	createGenericButton (doneButton, getLocalizedString('Done'), doneMouseClicked, 67);
	doneMouseClicked();	
}

if (window.widget) {
	widget.onhide = onhide;
	widget.onshow = onshow;
}

// flip, info button
var flipShown = false;

var animation = {duration:0, starttime:0, to:1.0, now:0.0, from:0.0, firstElement:null, timer:null};

function mousemove (event)
{
	if (!flipShown)
	{
		if (animation.timer != null)
		{
			window.clearInterval (animation.timer);
			animation.timer  = null;
		}
		
		var starttime = (new Date).getTime() - 13;
		
		animation.duration = 500;
		animation.starttime = starttime;
		animation.firstElement = document.getElementById ('flip');
		animation.timer = window.setInterval ("animate();", 13);
		animation.from = animation.now;
		animation.to = 1.0;
		animate();
		flipShown = true;
	}
}

function mouseexit (event)
{
	if (flipShown)
	{
		// fade in the info button
		if (animation.timer != null)
		{
			window.clearInterval (animation.timer);
			animation.timer  = null;
		}
		
		var starttime = (new Date).getTime() - 13;
		
		animation.duration = 500;
		animation.starttime = starttime;
		animation.firstElement = document.getElementById ('flip');
		animation.timer = window.setInterval ("animate();", 13);
		animation.from = animation.now;
		animation.to = 0.0;
		animate();
		flipShown = false;
	}
}

function animate()
{
	var T;
	var ease;
	var time = (new Date).getTime();
		
	
	T = limit_3(time-animation.starttime, 0, animation.duration);
	
	if (T >= animation.duration)
	{
		window.clearInterval (animation.timer);
		animation.timer = null;
		animation.now = animation.to;
	}
	else
	{
		ease = 0.5 - (0.5 * Math.cos(Math.PI * T / animation.duration));
		animation.now = computeNextFloat (animation.from, animation.to, ease);
	}
	
	animation.firstElement.style.opacity = animation.now;
}

function limit_3 (a, b, c)
{
    return a < b ? b : (a > c ? c : a);
}

function computeNextFloat (from, to, ease)
{
    return from + (to - from) * ease;
}
